@@ -12,115 +12,6 @@ static SEXP build_return_array(SEXP nodes, short int *status,
  * Bang-Jensen and Gutin, page 13.
  */
 
-SEXP is_dag_acyclic(SEXP arcs, SEXP nodes, SEXP return_nodes, SEXP debug) {
-
-SEXP amat;
-int i = 0, j = 0, z = 0;
-int nrows = LENGTH(nodes);
-int check_status = nrows;
-int check_status_old = nrows;
-int rowsums, colsums;
-int *a = NULL, *debuglevel = NULL;
-short int *status = NULL;
-
-  /* dereference the debug parameter. */
-  debuglevel = LOGICAL(debug);
-
-  /* build the adjacency matrix from the arc set.  */
-  if (*debuglevel > 0)
-    Rprintf("* building the adjacency matrix.\n");
-
-  PROTECT(amat = arcs2amat(arcs, nodes));
-  a = INTEGER(amat);
-
-  /* allocate and initialize the status array. */
-  status = allocstatus(nrows);
-
-  if (*debuglevel > 0)
-    Rprintf("* checking whether the directed graph is acyclic.\n");
-
-  /* even in the worst case scenario at least two nodes are marked as
-   * good at each iteration, so even ceil(nrows/2) iterations should be
-   * enough. */
-  for (z = 0; z < nrows; z++) {
-
-    if (*debuglevel > 0)
-      Rprintf("* beginning iteration %d.\n", z + 1);
-
-    for (i = 0; i < nrows; i++) {
-
-      /* skip known-good nodes. */
-      if (status[i] == GOOD) continue;
-
-      /* reset and update row and column totals. */
-      rowsums = colsums = 0;
-
-      /* compute row and column totals for the i-th node. */
-      for (j = 0; j < nrows; j++) {
-
-        rowsums += a[CMC(i, j, nrows)];
-        colsums += a[CMC(j, i, nrows)];
-
-      }/*FOR*/
-
-      if (*debuglevel > 0)
-        Rprintf("  > checking node %s (%d child(ren), %d parent(s)).\n",
-          NODE(i), rowsums, colsums);
-
-      /* if either total is zero, the node is either a root node or a
-       * leaf node, and is not part of any cycle. */
-      if ((rowsums == 0) || (colsums == 0)) {
-
-        if (*debuglevel > 0)
-          Rprintf("  @ node %s is cannot be part of a cycle.\n", NODE(i));
-
-        for (j = 0; j < nrows; j++)
-          a[CMC(i, j, nrows)] = a[CMC(j, i, nrows)] = 0;
-
-        /* mark the node as good. */
-        status[i] = GOOD;
-        check_status--;
-
-      }/*THEN*/
-
-    }/*FOR*/
-
-    /* at least three nodes are needed to have a cycle. */
-    if (check_status < 3) {
-
-      if (*debuglevel > 0)
-        Rprintf("@ at least three nodes are needed to have a cycle.\n");
-
-      UNPROTECT(1);
-      return build_return_array(nodes, status, nrows, check_status, return_nodes);
-
-    }/*THEN*/
-
-    /* if there are three or more bad nodes and there was no change in
-     * the last iteration, the algorithm is stuck on a cycle. */
-    if (check_status_old == check_status) {
-
-      if (*debuglevel > 0)
-        Rprintf("@ no change in the last iteration.\n");
-
-      UNPROTECT(1);
-      return build_return_array(nodes, status, nrows, check_status, return_nodes);
-
-    }/*THEN*/
-    else
-      check_status_old = check_status;
-
-  }/*FOR*/
-
-  UNPROTECT(1);
-  return build_return_array(nodes, status, nrows, check_status, return_nodes);
-
-}/*IS_DAG_ACYCLIC*/
-
-/*
- * An extension of the previous algorithm to the mixed graph case.
- */
-
 SEXP is_pdag_acyclic(SEXP arcs, SEXP nodes, SEXP return_nodes, SEXP debug) {
 
 int i = 0, j = 0, z = 0;
